@@ -354,11 +354,9 @@ class SparseAttention(Attention):
         q, k, v = (self.to_q(x), *self.to_kv(x).chunk(2, dim = -1))
         q, k, v = map(lambda t: rearrange(t, 'b n (h d) -> b h n d', h = h), (q, k, v))
 
-        dots = einsum('b h i d, b h j d -> b h i j', q, k) * self.scale
-
         key_pad_mask = None
         if exists(mask):
-            key_pad_mask = ~mask
+            key_pad_mask = repeat(~mask, 'b n -> b h n', h = h)
 
         out = self.attn_fn(q, k, v, key_padding_mask = key_pad_mask)
         out = rearrange(out, 'b h n d -> b n (h d)')
@@ -653,7 +651,7 @@ class Alphafold2(nn.Module):
                 raise ValueError(f'cannot find attention type {attn_type}')
 
             layers.append(nn.ModuleList([
-                prenorm(InterceptAxialAttention(tensor_slice, AxialAttention(dim = dim, template_axial_attn = template_axial_attn, seq_len = (max_seq_len, max_seq_len, max_num_templates), heads = heads, dim_head = dim_head, dropout = attn_dropout, sparse_attn = sparse_self_attn))),
+                prenorm(InterceptAxialAttention(tensor_slice, AxialAttention(dim = dim, template_axial_attn = template_axial_attn, seq_len = max_seq_len, heads = heads, dim_head = dim_head, dropout = attn_dropout, sparse_attn = sparse_self_attn))),
                 prenorm(InterceptFeedForward(tensor_slice, ff = FeedForward(dim = dim, dropout = ff_dropout))),
                 prenorm(AxialAttention(dim = dim, seq_len = max_seq_len, heads = heads, dim_head = dim_head, dropout = attn_dropout, tie_row_attn = msa_tie_row_attn)),
                 prenorm(FeedForward(dim = dim, dropout = ff_dropout)),
@@ -777,7 +775,7 @@ class Alphafold2(nn.Module):
 
             msa_shape = m.shape
             m = rearrange(m, 'b m n d -> b (m n) d')
-            
+
             # get msa_mask to all ones if none was passed
             msa_mask = default(msa_mask, torch.ones_like(msa).bool())
 
@@ -909,8 +907,8 @@ class Alphafold2(nn.Module):
         chain_mask = (mask.unsqueeze(-1) * cloud_mask)
         flat_chain_mask = rearrange(chain_mask, 'b l c -> b (l c)')
 
-        bb_mask = rearrange(chain_mask[:, :, :self.num_backbone_atoms], 'b l c -> b (l c)')
-        bb_mask_crossed = rearrange(bb_mask, 'b i -> b i ()') * rearrange(bb_mask, 'b j -> b () j')
+        bb_flat_mask = rearrange(chain_mask[..., :self.num_backbone_atoms], 'b l c -> b (l c)')
+        bb_flat_mask_crossed = rearrange(bb_flat_mask, 'b i -> b i ()') * rearrange(bb_flat_mask, 'b j -> b () j')
 
         # structural refinement
 
@@ -920,7 +918,8 @@ class Alphafold2(nn.Module):
         else:
             distances, weights = center_distogram_torch(distance_pred)
 
-        weights.masked_fill_(bb_mask_crossed, 0.)
+        # set unwanted atoms to weight=0 (like C-beta in glycine)
+        weights.masked_fill_( torch.logical_not(bb_flat_mask_crossed), 0.)
 
         coords_3d, _ = MDScaling(distances, 
             weights = weights,
